; #######################################################################
;
;                               smqtoa
;
; Procedure for converting a 64-bit signed integer from memory to ASCII
; without any leading "0"
;
; Return:   Null-terminated ASCII string in memory
;           EAX = address of first byte of the string
;
; Usage:    invoke smqtoa,lpQW,lpBuf
;
;   where:  
;   lpQW  = pointer to the location of the qword in memory
;   lpBuf = pointer to a memory buffer for the null-terminated ASCII
;           string (a 22-byte buffer is sufficient for the largest of
;           numbers, i.e. 20 digits maximum, sign and the terminating 0)
;
; EAX, ECX and EDX are trashed. EBX, ESI and EDI are preserved.
;
; NOTE: This procedure cannot determine if the data pointed to by
;       lpQW is a valid signed QWORD INTEGER. It is the responsibility
;       of the programmer to insure that. No error code can be returned.
;
;                       by Raymond Filiatreault
;                            Otober 2009
;                            MASM  syntax
;
; The "m" in the procedure name is to indicate that this procedure is
; based on the principal of multiplying by the reciprocal of 10 instead
; of dividing by 10, i.e. the use of "magic numbers".
;
; #######################################################################

      .686                   ; minimum processor needed for 32 bit
      .model flat, stdcall   ; FLAT memory model & STDCALL calling
      option casemap :none   ; set code to case sensitive

; #######################################################################

;smqtoa PROTO :DWORD, :DWORD

.code

smqtoa proc uses ebx esi edi lpsrc:DWORD, lpdest:DWORD

LOCAL x10 :DWORD
LOCAL r10L :DWORD
LOCAL r10H :DWORD
LOCAL t1 :DWORD
LOCAL t2 :DWORD

      mov   esi,lpsrc
      mov   edi,lpdest
      mov   ebx,[esi]
      mov   esi,[esi+4]
      add   edi,21
      test  esi,esi
      push  esi
      .if   SIGN?
            not   esi
            not   ebx
            add   ebx,1
            adc   esi,0
      .endif
      mov   byte ptr[edi],0
      mov   r10L,0CCCCCCCDh
      mov   r10H,0CCCCCCCCh
      mov   x10,10
   @@:
      .if   esi != 0
            xor   ecx,ecx
            mov   eax,r10L
            mul   ebx
            mov   t1,edx
            mov   eax,r10H
            mul   ebx
            add   t1,eax
            adc   edx,0
            mov   t2,edx
            mov   eax,r10L
            mul   esi
            add   t1,eax
            adc   t2,edx
            adc   ecx,0
            mov   eax,r10H
            mul   esi
            add   eax,t2
            adc   edx,ecx
            mov   ecx,t1
            shrd  ecx,eax,3
            shrd  eax,edx,3
            inc   ecx
            shr   edx,3
            mov   ebx,eax
            mov   esi,edx
            mov   eax,ecx
            mul   x10
            add   dl,30h
            dec   edi
            mov   [edi],dl
            jmp   @B
      .endif
   @@:
      mov   eax,ebx
      mul   r10L
      shrd  eax,edx,3
      shr   edx,3
      mov   ebx,edx
      inc   eax
      mul   x10
      dec   edi
      add   dl,30h
      mov   [edi],dl
      test  ebx,ebx
      jnz   @B
      pop   esi
      test  esi,esi
      jns   @F
      dec   edi
      mov   byte ptr[edi],"-"
   @@:
      mov   eax,edi
      ret

smqtoa endp

end
