; #######################################################################
;
;                               umdtoa
;
; Procedure for converting a 32-bit unsigned integer to ASCII without
; any leading "0"
;
; Return:   Null-terminated ASCII string in memory
;           EAX = address of first byte of the string
;
; Usage:    invoke umdtoa,src,lpBuf
;
;   where:  
;   src   = dword to be converted
;   lpBuf = pointer to a memory buffer for the null-terminated ASCII
;           string (an 11-byte buffer is sufficient for the largest of
;           numbers, i.e. 10 digits maximum, and the terminating 0)
;
; EAX, ECX and EDX are trashed. EBX, ESI and EDI are preserved.
;
; NOTE: This procedure cannot determine if the source dword
;       is a valid unsigned DWORD INTEGER. It is the responsibility
;       of the programmer to insure that. No error code can be returned.
;
;                       by Raymond Filiatreault
;                            Otober 2009
;                            MASM  syntax
;
; The "m" in the procedure name is to indicate that this procedure is
; based on the principal of multiplying by the reciprocal of 10 instead
; of dividing by 10, i.e. the use of "magic numbers".
;
; #######################################################################

      .686                   ; minimum processor needed for 32 bit
      .model flat, stdcall   ; FLAT memory model & STDCALL calling
      option casemap :none   ; set code to case sensitive

; #######################################################################

;umdtoa PROTO :DWORD, :DWORD

.code

umdtoa proc uses ebx edi src:DWORD, lpdest:DWORD

      mov   edi,lpdest
      mov   eax,src
      add   edi,11
      mov   ecx,0CCCCCCCDh
      mov   ebx,10
      mov   byte ptr[edi],0
   @@:
      mul   ecx
      shrd  eax,edx,3
      shr   edx,3
      inc   eax
      push  edx
      mul   ebx
      dec   edi
      add   dl,30h
      mov   [edi],dl
      pop   eax
      test  eax,eax
      jnz   @B
      mov   eax,edi
      ret

umdtoa endp

end
